"""
Here, we test the backend functionality of the library
and see if it throws a fit.
"""

from bkm10_lib import backend, formalism

backend.set_backend("numpy")
assert np.allclose(...)

backend.set_backend("tensorflow")
assert tf.reduce_all(...)


# tests/test_unpolarized.py

# (X): Native Library | unittest:
import unittest

# (X): External Library | NumPy:
import numpy as np

# (X): Self-Import | BKM10Inputs:
from bkm10_lib.inputs import BKM10Inputs

# (X): Self-Import | CFFInputs:
from bkm10_lib.cff_inputs import CFFInputs

# (X): Self-Import | DifferentialCrossSection
from bkm10_lib.core import DifferentialCrossSection


# (X): Define a class that inherits unittest's TestCase:
class TestUnpolarizedCoefficients(unittest.TestCase):

    def setUp(self):
        self.test_kinematics = BKM10Inputs(
            lab_kinematics_k = 5.75,
            squared_Q_momentum_transfer = 1.82,
            x_Bjorken = 0.34,
            squared_hadronic_momentum_transfer_t = -0.17)

        self.test_cff_inputs = CFFInputs(
            compton_form_factor_h = complex(-0.897, 2.421),
            compton_form_factor_h_tilde = complex(2.444, 1.131),
            compton_form_factor_e = complex(-0.541, 0.903),
            compton_form_factor_e_tilde = complex(2.207, 5.383))
        
        



"""
Entry point for the BKM10Inputs dataclass.
"""

# (1): Import the specialized `dataclass` library:
from dataclasses import dataclass

# (2): Define the dataclass right away:
@dataclass
class BKM10Inputs:
    """
    Welcome to the `BKKM10Inputs` dataclass! 

    ## Description:
    We use this dataclass to handle the passage of the 
    "kinematic settings" that parametrize the DVCS cross section 
    according to the BKM10 formalism. That means, in order to
    obtain a cross section at the end of your calculation, you 
    must instantiate this dataclass.
    """

    # (1): Q^{2}: photon virtuality:
    squared_Q_momentum_transfer: float

    # (2): x_{B}: Bjorken's x:
    x_Bjorken: float

    # (3): t: hadron momentum transfer: (p - p')^{2}:
    squared_hadronic_momentum_transfer_t: float

    # (4): ...
    lab_kinematics_k: float



"""
# BKM10 Library
The fundamental package for computing (cross sections using the
BKM10 formalism)!

## Description:
A Python library to help nuclear physicists use the BKM formalism in predicting
cross-sections, asymmetries, and comparing GPD models.
"""

# (1): Expose the cross-section calculator class:
from .core import DifferentialCrossSection

# (2): Expose the BKM formalism class... in case people want that:
from .formalism import BKMFormalism

# (3): Expose the dataclass `CFFInputs`... for getting familiar with it:
from .cff_inputs import CFFInputs

# (4): Expose the dataclass `BKM10Inputs` in order to evaluate the cross section:
from .inputs import BKM10Inputs

# (5): Expose the backend: This is important if the user is using TensorFlow:
from .backend import math
